local BuildPetTalentInfo
do
	local GetNumTalentTabs = GetNumTalentTabs
	local GetTalentTabInfo = GetTalentTabInfo
	local GetNumTalents = GetNumTalents
	local GetTalentInfo = GetTalentInfo
	local GetTalentPrereqs = GetTalentPrereqs
	local TOOLTIP_TALENT_LEARN = TOOLTIP_TALENT_LEARN

	local function FillPrereqs(...)
		if select("#", ...) < 3 then return end
		local table = {}
		for i = 1, select("#", ...), 3 do
			local row, column = select(i, ...)
			table[#table + 1] = { row = row, column = column }
		end
		return table
	end

	function BuildPetTalentInfo()
		local G = LibStub("LibGratuity-3.0")

		if GetNumTalentTabs(nil, true) ~= 1 then return end

		local name, icon, _, background = GetTalentTabInfo(1, nil, true)
		local result = {
			info = {
				name = name,
				icon = icon,
				background = background,
			},
			talents = {}
		}
		local talents = result.talents
		for index = 1, GetNumTalents(1, nil, true) do
			local name, icon, row, column, rank, ranks, exceptional = GetTalentInfo(1, index, nil, true)
			G:SetTalent(1, index, nil, true)
			local n = G:NumLines()
			local tips = G:GetLine(n)
			if tips == TOOLTIP_TALENT_LEARN then
				tips = G:GetLine(n - 1)
			end

			talents[index] = {
				info = {
					name = name,
					icon = icon,
					row = row,
					column = column,
					ranks = ranks,
					exceptional = exceptional,
					prereqs = FillPrereqs(GetTalentPrereqs(1, index, nil, true)),
					tips = tips,
				},
			}
		end

		return result
	end
end

local function FindTalentIndex(table, row, column)
	for index, talent in ipairs(table) do
		if talent.info.row == row and talent.info.column == column then
			return index
		end
	end
end

local function FindPrereqsSource(info)
	for tab, tree in ipairs(info) do
		for index, talent in ipairs(tree.talents) do
			local p = talent.info.prereqs
			if p then
				for _, req in ipairs(p) do
					if req.source then
						return
					else
						req.source = FindTalentIndex(tree.talents, req.row, req.column)
					end
				end
			end
		end
	end
end

function Talented:GetPetInfo(class)
	if not self.pet_talents then
		if self:DataAddonLoaded() and Talented_PetData then
			self.pet_talents = Talented_PetData
		else
			self.pet_talents = {}
		end
	end
	local info = self.pet_talents[class]
	if info then
		FindPrereqsSource(info)
	elseif class == self:GetPetClass() then
		info = BuildPetTalentInfo()
		self.pet_talents[class] = info
	end
	if not info then
		self:ReportMissingTalents("PET")
	end
	return info
end

function Talented:GetPetClass()
	local _, icon = GetTalentTabInfo(1, nil, true)
	if icon == "Interface\\Icons\\Ability_Druid_Swipe" then
		return "FEROCITY"
	elseif icon == "Interface\\Icons\\Ability_Hunter_Pet_Bear" then
		return "TENACITY"
	elseif icon == "Interface\\Icons\\Ability_Hunter_CombatExperience" then
		return "CUNNING"
	end
end

if select(2, UnitClass("player")) ~= "HUNTER" then return end

function Talented:PET_TALENT_POINTS_CHANGED()
	local info
	local template = self.pet_current
	if not template then
		local class = self:GetPetClass()
		if not class then return end
		template = {
			pet = true,
			name = "Current",
			class = class,
		}
		self.pet_current = template
		info = self:GetPetInfo(class)
		if not info then
			error(("Unable to get talent for class %s, %s"):format(class, "pet"))
		end
	else
		info = self:GetPetInfo(template.class)
	end
	local total = 0
	for index in ipairs(info.talents) do
		local rank = select(5, GetTalentInfo(1, index, nil, true))
		template[index] = rank
		total = total + rank
	end
	self.pet_maxpoints = total + GetPetTalentPoints()
	if self.template == self.pet_current then
		self:UpdateTooltip()
	end
end
